/*
 * @(#)ProblemSetApplication.java  2.0  5 March 2002
 *
 * Copyright 2001,2002
 * College of Computer Science
 * Northeastern University
 * Boston, MA  02115
 *
 * This software may be used for educational purposes as long as
 * this copyright notice is retained intact at the top of all files.
 *
 * Should this software be modified, the words "Modified from 
 * Original" must be included as a comment below this notice.
 *
 * All publication rights are retained.  This software or its 
 * documentation may not be published in any media either in whole
 * or in part without explicit permission.
 *
 * Contact information:
 *   Richard Rasala    rasala@ccs.neu.edu
 *   Viera Proulx      vkp@ccs.neu.edu
 *   Jeff Raab         jmr@ccs.neu.edu
 * 
 * Telephone:          617-373-2462
 *
 * This software was created with support from Northeastern 
 * University and from NSF grant DUE-9950829.
 */

import edu.neu.ccs.gui.*;
import edu.neu.ccs.console.*;
import java.awt.*;
import javax.swing.*;
import java.lang.reflect.*;

/**
 * <P>This is a main application that sets up a console, a graphics window,
 * and installs a list of actions to select one of several tasks to run
 * from a given problem set.</P>
 *
 * @author Viera Proulx
 * @author Richard Rasala
 * @author Jason Jay Rodrigues
 */
public class ProblemSetApplication extends DisplayPanel implements ConsoleAware {
    		   
    /** The default buffer width. */
    protected static final int BUFFER_WIDTH  = 400;
    
    /** The default buffer height. */
    protected static final int BUFFER_HEIGHT = 400;
    
    /** The default graphics buffered panel. */
    protected BufferedPanel window = new BufferedPanel(BUFFER_WIDTH, BUFFER_HEIGHT);
    
    /** The buttons panel for the problem set. */
    protected ActionsPanel buttons = null;
    
    /** The maximum number of rows in the buttons panel. */
    protected final int MAXIMUM_ROWS = 20;
    
    /** The default problem set frame title. */
    protected String problemSetTitle = "Problem Set";
    		   
    /** The problem set. */
    protected ProblemSetClass problemSet = null;
    
    /** The exclosing JPTFrame of the application. */
    protected JPTFrame frame = null;
    
    /////////////////
    // Constructor //
    /////////////////
    
    /**
     * The default constructor which constructs the associated
     * problem set and its GUI.
     */
    public ProblemSetApplication() {
        buildProblemSet();
        buildButtons();
        buildGUI();
    }
    
    /////////////////////////////
    // Public Member Functions //
    /////////////////////////////
    
    public BufferedPanel getGraphicsWindow() {
        return window;
    }
    
    public void refresh() {
        frame.pack();
        frame.setLocation(NORTH_EAST);
    }
    
    ////////////////////////////////
    // Protected Member Functions //
    ////////////////////////////////
    
    /** Build the problem set. */
    protected void buildProblemSet() {
        // define the problem set object
        problemSet = new ProblemSetClass(this);
        
        // change the frame title
        if (problemSet.title != null)
            problemSetTitle = problemSet.title;
    }
    
    /** Build the buttons. */
    protected void buildButtons() {
		// create button panel for all tasks from the problem set
		buttons = new ActionsPanel(
		    getActionList(),
		    new TableLayout(MAXIMUM_ROWS, 1, 0, 0, CENTER, HORIZONTAL)
		);
		
        // add the Clear Window button
		buttons.addAction(new SimpleAction("Clear Window") 
		    { public void perform () { clearWindow(); } } );

        // add the Exit button
		buttons.addAction(new SimpleAction("Exit") 
		    { public void perform () { System.exit(0); } } );
		
		// uniformize the button sizes
		buttons.uniformizeSize();
    }
    
    /** Build the main panel. */
    protected void buildGUI() {
        // show the console window
        if (problemSet.showConsoleWindow) {
    		showConsole();
    		showConsoleMessage();
        }
        
        Display LHS = new Display(buttons, null, "Tasks");
        
        Display RHS = (problemSet.showGraphicsWindow)
            ? new Display(window,  null, "Graphics") 
            : null;
        
        TablePanel panel = new TablePanel(
            new Object[] { LHS, RHS }, HORIZONTAL, 5, 5, NORTH);
        
        add(panel);
    }
    
    /** Clear Window method for Clear Window action. */
    protected void clearWindow() {
        window.clearPanel();
        window.repaint();
    }
    
    /**
     * Return a list of the actions correponding to methods in
     * the ProblemSetClass that are public synchronized void
     * with zero arguments but not static.  This list may have
     * null's.
     */
    protected Action[] getActionList(){
        // extract methods
        Method[] methodList = problemSet.getClass().getDeclaredMethods();
        int length = methodList.length;
        
        // build action list
        Action[] ActionList = new Action[length];
        
        for (int i = 0; i < length; i++)
            if (isProperMethod(methodList[i]))
                ActionList[i] = makeActionFromMethod(methodList[i]);
        
        return ActionList;
    }
    
    /** Make a ThreadedAction from the given method and name string. */
    protected Action makeActionFromMethod(final Method method) {
        if (method == null)
            return null;
        
        final String name = method.getName();
        
        return new ThreadedAction(
            new SimpleAction(name) {
                public void perform () { 
                    console.out.println("Run: " + name + "\n");

                    try{
                        method.invoke(problemSet, null);
                        }
                    catch (IllegalAccessException illAccEx){
                        // should never happen
                        // method is public
                        console.err.print  ("IllegalAccessException: ");
                        console.err.println("" + illAccEx); 
                        console.err.println("In: " + name + "\n");
                    }
                    catch (IllegalArgumentException illArgEx){
                        // should never happen
                        // method has zero arguments
                        console.err.print  ("IllegalArgumentException: ");
                        console.err.println("" + illArgEx);
                        console.err.println("In: " + name + "\n");
                    }
                    catch (InvocationTargetException invTarEx){
                        // may happen if exception thrown inside method
                        console.err.print  ("InvocationTargetException: ");
                        console.err.println("" + invTarEx);
                        console.err.println("In: " + name + "\n");
                        
                        Throwable target = invTarEx.getTargetException();
                        
                        console.err.println("Target Stack Trace:");
                        target.printStackTrace(console.err);
                        console.err.println();
                    }
                    
                    console.out.println("End: " + name + "\n\n");
                }
            });    
    }

    //////////////////
    // Main Program //
    //////////////////

    public static void main(String[] args) {
        ProblemSetApplication application
            = new ProblemSetApplication();
		
		application.frame
		    = JPTFrame.createQuickJPTFrame(
		        application.problemSetTitle,
		        application,
		        new TableLayout(1, 1, NORTH),
		        NORTH_EAST
		    );
    }

    //////////////////////
    // Static Functions //
    //////////////////////
    
    /**
     * Return true if the method is public synchronized void
     * with zero arguments but not static.
     */
    protected static boolean isProperMethod(Method method) {
        if (method == null)                             // missing method
            return false;
        
        if (method.getReturnType() != void.class)       // not void
            return false;
        
        if (method.getParameterTypes().length > 0)      // has parameters
            return false;
        
        int modifiers = method.getModifiers();
        
        boolean OK =
            ((modifiers & Modifier.PUBLIC) != 0)        // public
          &&
            ((modifiers & Modifier.SYNCHRONIZED) != 0)  // synchronized
          &&
            ((modifiers & Modifier.STATIC) == 0);       // but not static
            
        return OK;
    }
    
    /** Show the console with color text. */
    protected static void showConsole() {
        console.setActivated(true);
		console.selectColorTextScheme();
	}
	
    /** Print opening console message. */
	protected static void showConsoleMessage() {
        console.out.println();
        console.out.println("Console is active.");
        console.out.println();        
	}
}
